#define  _GNU_SOURCE
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdint.h>


#define ISDIGIT(c) (c >= 0x30 && c <= 0x39)

inline uint64_t parse_number(const char *line, ssize_t j);


int main(int argc, char **argv) {
    FILE *input;
    char *line = NULL;
    size_t line_length = 0;

    if (argc == 1 || argv[1][0] == '-')
		input = stdin;
	else if ((input = fopen(argv[1], "r")) == NULL) {
		fprintf(stderr, "Cannot open file %s\n", argv[1]);
		return -1;
	}

    char **engine = NULL;
    ssize_t engine_line_size = 0;
    ssize_t engine_lines = 0;

    while (getline(&line, &line_length, input) != -1) {
        ssize_t len = strlen(line)-1;

        if (engine == NULL) {
            engine = (char **) calloc(len, sizeof(char*));
            engine_line_size = len;
        }

        engine[engine_lines] = (char*) calloc(len, sizeof(char));
        memcpy(engine[engine_lines], line, len);
        ++engine_lines;

        memset(line, 0, line_length);
    }

    uint64_t part_number = 0;

    for (ssize_t i = engine_lines-1; i >= 0; --i)
        for (ssize_t j = engine_line_size-1; j >= 0; --j) {
            if (engine[i][j] != '*')
                continue;

            uint64_t n = 1;
            int8_t p = 0;

            if (ISDIGIT(engine[i][j+1])) {
                n *= parse_number(engine[i], j+1);
                p++;
            }
            if (ISDIGIT(engine[i][j-1])) {
                n *= parse_number(engine[i], j-1);
                p++;
            }

            if (ISDIGIT(engine[i+1][j])) {
                n *= parse_number(engine[i+1], j);
                p++;
            } else {
                if (ISDIGIT(engine[i+1][j+1])) {
                    n *= parse_number(engine[i+1], j+1);
                    p++;
                }
                if (ISDIGIT(engine[i+1][j-1])) {
                    n *= parse_number(engine[i+1], j-1);
                    p++;
                }
            }

            if (ISDIGIT(engine[i-1][j])) {
                n *= parse_number(engine[i-1], j);
                p++;
            } else {
                if (ISDIGIT(engine[i-1][j+1])) {
                    n *= parse_number(engine[i-1], j+1);
                    p++;
                }
                if (ISDIGIT(engine[i-1][j-1])) {
                    n *= parse_number(engine[i-1], j-1);
                    p++;
                }
            }

            if (p >= 2)
                part_number += n;
        }

    for (ssize_t i = 0; i < engine_lines; ++i)
        free(engine[i]);
    free(engine);

    printf("%lu\n", part_number);

    return 0;
}

inline uint64_t parse_number(const char *line, ssize_t j) {
    uint64_t n = 0;
    uint64_t multiplier = 1;

    while (ISDIGIT(line[j+1])) j++;

    while (ISDIGIT(line[j])) {
        n += (line[j]&0xf) * multiplier;
        multiplier *= 10;
        j--;
    }

    return n;
}